__version__ = '1.0.0b0'

import cmd
import sys
import readline
import shlex
from argparse import ArgumentParser
from argparse import _SubParsersAction
from argparse import _HelpAction

import io
from contextlib import redirect_stdout
import traceback


################################################################################
class CommandArgumentParser(ArgumentParser):
    # Preventing to execute exit command after help or error
    def error(self, message):
        self.print_help(sys.stderr)

    def exit(self, status=0, message=None):
        if message:
            self._print_message(message, sys.stderr)


################################################################################
class ArgumentCmd(cmd.Cmd):
    argument_parser = None

    # ==========================================================================
    def __init__(self):
        super(ArgumentCmd, self).__init__()

    # ==========================================================================
    def _do_command(self, line, *args, **kwargs):
        parser = self.argument_parser
        prog = kwargs["prog"].strip()
        try:
            command = f'{prog} {line}'.strip()
            spec = parser.parse_args(shlex.split(command))
            if 'func' not in spec:
                return
            if any(x in command for x in ('-h', '--help')):
                return
            spec.func(spec)
        except TypeError:
            # parse_args raise exception when wrong input.
            return
        except Exception as e:
            traceback.print_exc()
            # Todo: Do something for errors
            return

    # ==========================================================================
    def _complete_command(self, text, line, start_index, end_index, **kwargs):
        parser = self.argument_parser()
        return ArgumentCmd.get_complete_list(line, parser).append(None)

    # ==========================================================================
    def emptyline(self):
        pass

    # ==========================================================================
    def complete(self, text, state):
        if state == 0:
            origline = readline.get_line_buffer()
            line = origline.lstrip()
            command, args, foo = self.parseline(line)
            if command == '':
                result = self.completedefault
            else:
                result = ArgumentCmd.get_complete_list(
                    shlex.split(line), self.argument_parser)
                result = [x + ' ' for x in result]
            result.append(None)
            self.completion_matches = result
        try:
            return self.completion_matches[state]
        except IndexError:
            return None

    # ==========================================================================
    @staticmethod
    def get_complete_list(line, parser, result=None):
        if None is result:
            result = list()
        word = ''
        if line:
            word = line.pop(0)

        for action in parser._actions:
            if isinstance(action, _HelpAction):
                continue
            if isinstance(action, _SubParsersAction):
                if not word:
                    return [x for x in action.choices.keys()]
                rc = [x for x in action.choices.keys() if x.startswith(word)]
                if word not in rc:
                    return rc
                return ArgumentCmd.get_complete_list(
                    line, action.choices[word], list())

        return result

    # ==========================================================================
    @staticmethod
    def get_empty_func(_type, action, parser):
        def do(self, line):
            self._do_command(self, line, prog=action.prog)

        def complete(self, text, line, start_index, end_index):
            return self._complete_command(
                self, text, line, start_index, end_index)

        f = {'do': do, 'complete': complete}
        return f[_type]

    # ==========================================================================
    @classmethod
    def _add_functions(cls, actions, parser):
        for action in actions.choices.values():
            with io.StringIO() as buf, redirect_stdout(buf):
                print(action.print_usage())
                output = buf.getvalue()

            for _type in ('do',):
                command = ArgumentCmd.get_empty_func(
                    _type, action=action, parser=parser)
                command.__doc__ = output
                command.__name__ = f"{_type}_{action.prog.split()[-1].strip()}"
                setattr(cls, command.__name__, classmethod(command))

    # ==========================================================================
    @classmethod
    def _add_command(cls, parser):
        for action in parser._actions:
            if isinstance(action, _HelpAction):
                continue
            if isinstance(action, _SubParsersAction):
                ArgumentCmd._add_functions(action, parser)

    # ==========================================================================
    @classmethod
    def set_cli_parser(cls, parser):
        cls.argument_parser = parser
        cls._add_command(parser)



import sys
import importlib.util
from argparse import ArgumentParser

from . import ArgumentCmd


################################################################################
def arg_parser():
    parser = ArgumentParser()
    parser.add_argument('cli_package', type=str)
    return parser


################################################################################
if __name__ == '__main__':
    arguments = arg_parser()
    arg_spec = arguments.parse_args()

    # Import user CLI package
    module_name = 'cli'
    spec = importlib.util.spec_from_file_location('cli', arg_spec.cli_package)
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    cli_parser = module.argument_parser()

    # Run CLI-Arguments
    cmd = ArgumentCmd
    cmd.set_cli_parser(cli_parser)
    my_cmd = cmd()
    my_cmd.cmdloop()



import io
import csv
import json
import functools
from tabulate import tabulate

__all__ = ['simple_table', 'convert_json_to_csv']


def convert_json_to_csv(f):
    @functools.wraps(f)
    def func(*args, **kwargs):
        result = f(*args, **kwargs)
        if not result:
            return None

        argspec = args[0]
        if argspec.raw_data:
            print(result)
            return None
        data = json.loads(result)
        try:
            buf = io.StringIO()
            dict_writer = csv.DictWriter(buf, fieldnames=data[0].keys(),
                                         delimiter=' ',
                                         quotechar='|',
                                         quoting=csv.QUOTE_MINIMAL)
            dict_writer.writeheader()
            dict_writer.writerows(data)
            buf.seek(0)
            reader = csv.reader(buf, delimiter=' ', quotechar='|',
                                quoting=csv.QUOTE_MINIMAL)
            return list(reader)
        except IOError:
            print("I/O error")

    return func


def simple_table(f):
    @functools.wraps(f)
    def func(*args, **kwargs):
        result = f(*args, **kwargs)
        if not result:
            return
        argspec = args[0]
        print(tabulate(result, headers='firstrow', showindex="always", tablefmt="orgtbl"))

    return func



import unittest
from cliparse import CommandArgumentParser,  ArgumentCmd
from cliparse import _SubParsersAction, _HelpAction
import pprint


class MyTestCase(unittest.TestCase):
    @staticmethod
    def argument_parser():
        parser = CommandArgumentParser(
            prog='',
            description='description',
            epilog='end of description',)

        sub_parser = parser.add_subparsers(dest='sub_parser')
        init_parser = sub_parser.add_parser('init', help='Initialize database')
        init_parser.add_argument('-d', '--init-db', action='store_true',
                                 help='initialize database.')
        init_parser.add_argument('-m', '--dummy-members', action='store_true',
                                 help='insert dummy members.')
        init_parser.add_argument('-b', '--dummy-books', action='store_true',
                                 help='insert dummy books.')
        init_parser.add_argument('-r', '--dummy-rental', action='store_true',
                                 help='insert dummy rental.')

        init_subparser = init_parser.add_subparsers(dest='init_subparser', help='init_sub')
        init_sub = init_subparser.add_parser('book', help='About Book')
        init_sub.add_argument('-a', '--about', help='about sub')

        init_sub = init_subparser.add_parser('book2', help='About Book')
        init_sub.add_argument('-a', '--about', help='about sub')

        run_app = sub_parser.add_parser('server', help='Run api server')
        run_app.add_argument('-a', '--address', default='localhost',
                             help='host address')
        run_app.add_argument('-p', '--port', type=int, default=5000,
                             help='port')
        run_app.add_argument('-d', '--debug', action='store_true')

        return parser

    def setUp(self) -> None:
        self.parser = self.argument_parser()

    def test0010_parser(self):
        # add_command('a', parser=self.parser)
        pass

    def test0020_subparser(self):
        self.parser.parse_args('-h'.split())

    def test0100_get_complete_list(self):
        line = ''.split()
        parser = self.argument_parser()
        print(ArgumentCmd.get_complete_list(line, parser))


if __name__ == '__main__':
    unittest.main()



import json
from cliparse import CommandArgumentParser
from cliparse.viewer.simple_table import simple_table, convert_json_to_csv

books = [
    {
        "author": "John Doe",
        "title": "How to learn speaking English",
        "publisher": "Magic House"
    },
    {
        "author": "George Orwell",
        "title": "1984",
        "publisher": "Motihari"
    }
]


def initialize_db(argspec):
    pass


@simple_table
@convert_json_to_csv
def book_show(argspec):
    return json.dumps(books, indent=4, ensure_ascii=False)


def book_insert(argspec):
    book = dict(title=argspec.title,
                author=argspec.author,
                publisher=argspec.publisher)
    books.append(book)


def book_update(argspec):
    try:
        book = books[argspec.index]
    except IndexError:
        print('You typed the wrong index number. Check and try again ...')
        return

    if argspec.title:
        book['title'] = argspec.title
    if argspec.author:
        book['author'] = argspec.author
    if argspec.publisher:
        book['publisher'] = argspec.publisher
    books[argspec.index] = book
    return


def book_delete(argspec):
    index = argspec.index
    index = index.sort()
    index.reverse()
    try:
        for i in index:
            books.pop(i)
    except IndexError:
        print(f'Out of index. Check and try again ...')
        return


def argument_parser():
    parser = CommandArgumentParser(
        prog='',
        description='description',
        epilog='end of description', )
    sub_parser = parser.add_subparsers(dest='sub_parser')

    # Setting
    # ==========================================================================
    setting_parser = sub_parser.add_parser('setting', help='Setting Command')

    # Setting - DB Initializing
    # ==========================================================================
    setting_init_db_parser = setting_parser.add_subparsers(
        dest='init', help='Initialize the database')

    init_db_parser = setting_init_db_parser.add_parser(
        name='initialize-db', help='Initialize database')
    init_db_parser.add_argument('-d', '--init-db', action='store_true',
                                help='initialize database.')
    init_db_parser.add_argument('-m', '--dummy-members', action='store_true',
                                help='insert dummy members.')
    init_db_parser.add_argument('-b', '--dummy-books', action='store_true',
                                help='insert dummy books.')
    init_db_parser.add_argument('-r', '--dummy-rental', action='store_true',
                                help='insert dummy rental.')
    init_db_parser.set_defaults(func=initialize_db)

    # Setting - DB Server Setting
    # ==========================================================================
    server_setting_parser = setting_init_db_parser.add_parser(
        name='server', help='Setting server')
    server_setting_parser.add_argument('-a', '--address', default='localhost',
                                       help='host address')
    server_setting_parser.add_argument('-p', '--port', type=int, default=5000,
                                       help='port')

    # ==========================================================================

    # Book Rental Manager
    # ==========================================================================
    manager_parser = sub_parser.add_parser('manager', help='Manager Command')

    manager_book_parser = manager_parser.add_subparsers(
        dest='book', help='Initialize the database')
    book_parser = manager_book_parser.add_parser('book', help='setting command')
    # viewer 옵션은 한 가지만 선택
    viewer_group = book_parser.add_mutually_exclusive_group()
    viewer_group.add_argument('-r', '--raw-data', action='store_true',
                              help='show the data as raw')
    viewer_group.add_argument('-t', '--simple-table', action='store_true',
                              help='show the data with simple table')
    book_parser.set_defaults(func=book_show)

    # Create
    book_crud_parser = book_parser.add_subparsers(dest='book_insert')

    create_parser = book_crud_parser.add_parser('insert')
    create_parser.add_argument('title', type=str)
    create_parser.add_argument('author', type=str)
    create_parser.add_argument('publisher', type=str)
    create_parser.set_defaults(func=book_insert)

    # Update
    update_parser = book_crud_parser.add_parser('update')
    update_parser.add_argument('index', type=int)
    update_parser.add_argument('-t', '--title', type=str)
    update_parser.add_argument('-a', '--author', type=str)
    update_parser.add_argument('-p', '--publisher', type=str)
    update_parser.set_defaults(func=book_update)

    delete_parser = book_crud_parser.add_parser('delete')
    delete_parser.add_argument('index', type=int, nargs='*',
                               help='index number(s) that you want to delete')
    delete_parser.set_defaults(func=book_delete)
    return parser


__all__ = ['argument_parser']

if '__main__' == __name__:
    parse = argument_parser()
    spec = parse.parse_args()
    if spec.func:
        spec.func(spec)



from setuptools import setup, find_packages
from cliparse import __version__


LONG_DESCRIPTION = open("README.md", "r", encoding="utf-8").read()

setup(
    name="cliparse",
    version=__version__,
    author="Duk Kyu Lim",
    author_email="hong18s@gmail.com",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    description='CLI Framework with Argparse',
    url="https://github.com/ravenkyu/cliparse",
    license="MIT",
    keywords="cli",
    install_requires=[
        'tabulate',
    ],
    packages=find_packages(),
    package_data={},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    zip_safe=False,
)
